{-# LANGUAGE Arrows, CPP #-}
module Automata where

#if __GLASGOW_HASKELL__ >= 610
import Prelude hiding ((.), id)
import Control.Category
#endif
import Control.Arrow

-- -----------------------------------------------------------

-- Mealy Automata
newtype Automata i o = Automata{ next :: i -> (Automata i o, o) }

#if __GLASGOW_HASKELL__ >= 610
instance Category Automata where
    id = Automata $ \x -> (id, x)
    f . g  = Automata $ \x ->
      case next g x of
      (g', y) ->
          case next f y of
          (f', z) -> (f' . g', z)
#endif

instance Arrow Automata where
#if __GLASGOW_HASKELL__ < 610
    g >>> f = Automata $ \x ->
      case next g x of
      (g', y) ->
          case next f y of
          (f', z) -> (g' >>> f', z)
#endif
    arr f = let a = Automata $ \x -> (a, f x) in a
    first f = Automata $ \(x, y) ->
      case next f x of
        (f', z) -> (first f', (z, y))

instance ArrowLoop Automata where
    loop f = Automata $ \x ->
      let (f', (z,y)) = next f (x,y)
      in (loop f', z)

constA :: b -> Automata a b
constA b = arr (\_ -> b)

-- followed by
fby :: a -> Automata a a
fby x = Automata $ \y -> (fby y, x)

diff :: Num a => Automata a (Maybe a)
diff = proc x -> do
  x' <- fby Nothing -< Just x
  returnA -< fmap (x -) x'

-- -----------------------------------------------------------

newtype Task a b c = Task{ runTask :: Automata a (Either b c) }

instance Monad (Task a b) where
  return x = Task $ arr (\_ -> Right x)
  -- 綺麗に書けなかった
  m >>= f = Task $ Automata $ \a ->
    case next (runTask m) a of
      (am, Left b) -> (runTask (Task am >>= f), Left b)
      (_, Right c) -> next (runTask (f c)) a 

mkTask :: Automata a (b, Maybe c) -> Task a b c
mkTask a = Task (a >>> arr f)
  where
    f :: (b, Maybe c) -> Either b c
    f (b, Nothing) = Left b
    f (_, Just c)  = Right c

runTask_ :: Task a b c -> Automata a b
runTask_ t = runTask t >>> arr (either id (const undefined))

constT :: b -> Task a b c
constT b = Task $ arr (\_ -> Left b)

snapT :: Task a b a
snapT = Task $ arr (\a -> Right a)

output :: b -> Task a b ()
output b = Task $ Automata $ \_ -> (constA (Right ()), Left b)

outputs :: [b] -> Task a b ()
outputs xs = mapM_ output xs

timeOut :: Task a b c -> Int -> Task a b (Maybe c)
timeOut task n = Task $ proc a -> do
  rec i <- fby n -< i-1
  x <- runTask task -< a
  returnA -< if i <= 0 then Right Nothing else fmap Just x

abortWhen :: Task a b c -> Automata a (Maybe d) -> Task a b (Either c d)
abortWhen task cond = Task $ proc a -> do
  x <- cond -< a
  y <- runTask task -< a
  returnA -< case x of
    Just d -> Right (Right d)
    Nothing -> fmap Left y

abortWhen_ :: Task a b c -> Automata a Bool -> Task a b (Maybe c)
abortWhen_ task cond = do
  x <- task `abortWhen` (cond >>> arr (\f -> if f then Just () else Nothing))
  case x of
    Right _ -> return Nothing
    Left c  -> return (Just c)

dmapTask :: (a -> a') -> (b' -> b) -> Task a' b' c -> Task a b c
dmapTask f g t = Task $ arr f >>> runTask t >>> arr h
  where
    h (Left b') = Left (g b')
    h (Right c) = Right c
