{-# LANGUAGE Arrows #-}
module Hohmann (controller, vmtrace) where

import Control.Monad
import Control.Arrow
import Data.Maybe
import Data.Complex
import qualified Data.IntMap as IM
import System.IO
import Text.Printf

import Physics
import OrbitVM
import Automata
import Controller
import Address

controller :: Controller
controller = proc i -> do
  p <- pos -< i
  let r1 = magnitude p
  r2 <- targetRadius -< i
  rec 
    v <- velocity -< (p, dv)
    dv <- runTask_ task -< (v,r1,r2)
  returnA -< dv

task :: Task (Maybe V, Double, Double) DV ()
task = do
  (v, r1, r2) <- snapT
  case v of
    Nothing -> output 0 >> task
    Just v -> do
      let
        theta1 = phase v
        theta2 = theta1 + pi
        dv1 = sqrt (mu / r1) * (sqrt (2 * r2 / (r1+r2)) - 1)
        dv2 = sqrt (mu / r2) * (1 - sqrt (2 * r1 / (r1+r2)))
        t_h = round (pi * sqrt ((r1+r2)**3 / (8*mu)))
      output (mkPolar dv1 theta1)
      wait (t_h - 1) -- 1マイナスしているのは加速に既に1秒費やしているので
      output (mkPolar dv2 theta2)
      constT 0

vmtrace :: TimeStep -> SensorData -> IO ()
vmtrace t o =
  when (t `mod` 1000 == 0) $ do
    let pos = (- (o IM.! addrX)) :+ (- (o IM.! addrY)) -- 地球からの相対座標
    printf "t=%d, radius=%f, targetRadius=%f\n" t (magnitude pos) (o IM.! addrTargetRadius)
    hFlush stdout
