{-# LANGUAGE Arrows, CPP #-}
module Main where

#define ENABLE_VIEWER

import Control.Monad
import Control.Concurrent (forkIO, yield)
import qualified Data.IntMap as IM
import Data.IORef
import System.IO
import System.Environment
import Text.Printf

import OrbitVM
import Automata
import Controller
import Address
import OSF

#ifdef ENABLE_VIEWER
import SDLViewer
#endif

import qualified Hohmann
import qualified MeetAndGreet

-- -----------------------------------------------------------

main :: IO ()
main = do
  args <- getArgs
  let n = read $ head $ args ++ ["1001"]
  run n

run :: Int -> IO ()
run n = do
  vm <- newIOVM (bin n)
#ifdef ENABLE_VIEWER
  oref <- newIORef Nothing
  forkIO $ startViewer (n `div` 1000) (scale n) oref
#endif

  let trace0 =
        case n `div` 1000 of
          1 -> Hohmann.vmtrace
          2 -> MeetAndGreet.vmtrace
          _ -> \_ _ -> return ()
      trace t o = do
        trace0 t o
#ifdef ENABLE_VIEWER
        writeIORef oref (Just o)
        yield
#endif
  let init = IM.fromList [(addrConf, fromIntegral n)]
  log <- controlWithTrace trace vm init (controller n)
  writeOSF (show n ++ ".osf") teamID n log

bin :: Int -> String
bin n =
  case n `div` 1000 of
    1 -> "bin1.obf"
    2 -> "bin2.obf"
    3 -> "bin3.obf"
    4 -> "bin4.obf"

scale :: Int -> Double
scale 1001 = 500000
scale 1002 = 500000
scale 1003 = 500000
scale 1004 = 500000
scale 2001 =  50000
scale 2002 = 400000
scale 2003 =  50000
scale 2004 =  50000
scale 3001 =  50000
scale 3002 = 400000
scale 3003 = 200000
scale 3004 =  80000
scale 4001 = 300000
scale 4002 = 400000
scale 4003 = 400000
scale 4004 = 400000
scale _    = 100000

controller :: Int -> Controller
controller n =
    case n `div` 1000 of
      1 -> Hohmann.controller
      2 -> MeetAndGreet.controller
      _ -> constA 0

-- Team Sampou
teamID :: TeamID
teamID = 776
