module Physics where

import Data.Complex

type Pos = Complex Double
type V  = Complex Double
type DV = Complex Double -- ΔV

-- 重力定数
g :: Double
g = 6.67428 / 10**11

-- 地球の質量
me :: Double
me = 6.0 * 10**24

mu :: Double
mu = g * me

-- simulate x dv で x の1秒後の状態を計算
simulate :: (Pos, V) -> DV -> (Pos, V)
simulate (s, v) dv = seq s' $ seq v' $ (s', v')
  where
    s' = s + v * dt + (1/2) * (gg_s + dv) * dt**2
    v' = v + (dv + (gg_s + gg s') / 2) * dt
    gg_s = gg s
    gg s = mkPolar (g * me / (magnitude s)**2) (phase (- s))
    dt = 1
